#ifndef sformat_header_included_p
#define sformat_header_included_p
//
// Copyright 2008, Lowell Boggs Jr.
//
// This file or directory, containing source code for a computer program,
// is Copyrighted by Lowell Boggs, Jr.  987 Regency Drive, Lewisville
// TX (USA), 75067.  You may use, copy, modify, and distribute this
// source file without charge or obligation so long as you agree to
// the following:
//
//  1.  You must indemnify Lowell Boggs against any and all financial
//      obligations caused by its use, misuse, function, or malfunction.
//      Further, you acknowledge that there is no warranty of any kind,
//      whatsoever.
//
//  2.  You agree not to attempt to patent any portion of this original
//      work -- though you may attempt to patent your own extensions to
//      it if you so choose.
//
//  3.  You keep this copyright notice with the file and all copies
//      of the file and do not change it anyway except language translation.
//
// You are responsible for enforcing your own compliance with these
// conditions and may not use this source file if you cannot agree to the
// above terms and conditions.
//

#include <string>
#include <sstream>

//
//  This header file defines a global free function (actually a class that
//  looks like free function) that lets you convert anything to a string
//  using the stream output operator, <<, except that you don't have to
//  mess around with its syntax.  Further, this mechanism is as fast as
//  calling sprintf -- and more flexible.  
//
//  Note that the SFormat(t) interface is implemented as a class object
//  construction -- but it looks like a function call for all intents and
//  purposes.  Sformat is actually a typedef for Sformatter<void> -- so 
//  read its documentation below
//
//  Note that if you want to use std::hex, or some io manipulator, you have
//  include <iomanip> in your program.
//

template<class T> struct Sformatter; // declared as a template only so that I don't
                                     // have a sformat.cpp file -- let the linker
                                     // do the work for us lazy jerks.

typedef Sformatter<void> Sformat; // this is the class you should really use.

template<>
struct Sformatter<void>
  //
  //  This class acts like a function in that you use it like a function:
  //
  //    std::string tmp  = Sformat(something);
  //    std::string tmp1 = Sformat(hex, 40);
  //    std::string tmp2 = Sformat("fred", "bill");
  //
  //  An Sformat object is a generic string conversion tool.  The constructor
  //  is a template that lets you convert anything to a string using the global
  //  operator << -- but you don't have to mess with that syntax.
  //
  //  An Sformat object can be converted into either a string or a char const *.
  //
  //  However, it returns a static buffer -- so you can't string calls together
  //  like this:
  //
  //    cout << Sformat(a) << Sformat(b) << Sformat(c); // BAD! unpredictable!
  //
  //
{
public:

  operator char const *()        { return mostRecentValue_.c_str(); }
  operator std::string const &() { return mostRecentValue_; }

  typedef std::ios_base& (*Manipulator)(std::ios_base& _Iosbase);

private:



  static std::stringstream       stream;  // does all the work
  static std::string             mostRecentValue_;
  static std::ios_base::fmtflags formatFlags_;

  void resetStream() const 
  {
    static bool firstTime = true;

    if(!firstTime)
    {
      stream.seekp(std::ios::beg);    // start over
      stream.flags(formatFlags_);     // with default width, numeric base, etc
    }
  
    firstTime = false;


  }

public:

/// 1 parm calls //////////////////////////////////////////////

  template<class T>
  Sformatter(T const &t)
  {
    resetStream();

    stream << t << '\0';

    mostRecentValue_ = stream.str().c_str();
  }

/// 2 parm calls //////////////////////////////////////////////

  template<class T, class U>
  Sformatter(T const &t, U const &u)
  {
    resetStream();

    stream << t << u << '\0';

    mostRecentValue_ = stream.str().c_str();
  }

  template<class U>
  Sformatter(Manipulator m, U const &u)
  {
    resetStream();

    stream << m << u << '\0';

    mostRecentValue_ = stream.str().c_str();
  }


/// 3 parm calls //////////////////////////////////////////////

  template<class T, class U, class V>
  Sformatter(T const &t, U const &u, V const &v)
  {
    resetStream();

    stream << t << u << v << '\0';

    mostRecentValue_ = stream.str().c_str();
  }

  
  template<class U, class V>
  Sformatter(Manipulator m, U const &u, V const &v)
  {
    resetStream();

    stream << m << u << v << '\0';

    mostRecentValue_ = stream.str().c_str();
  }


  template<class T, class V>
  Sformatter(T const &t, Manipulator m, V const &v)
  {
    resetStream();

    stream << t << m << v << '\0';

    mostRecentValue_ = stream.str().c_str();
  }

/// 4 parm calls //////////////////////////////////////////////

  template<class T, class U, class V, class W>
  Sformatter(T const &t, U const &u, V const &v, W const &w)
  {
    resetStream();

    stream << t << u << v << '\0';

    mostRecentValue_ = stream.str().c_str();
  }

  template<class U, class V, class W>
  Sformatter(Manipulator m, U const &u, V const &v, W const &w)
  {
    resetStream();

    stream << m << u << v << '\0';

    mostRecentValue_ = stream.str().c_str();
  }

  template<class T, class V, class W>
  Sformatter(T const &t, Manipulator m, V const &v, W const &w)
  {
    resetStream();

    stream << t << m << v << '\0';

    mostRecentValue_ = stream.str().c_str();
  }


  template<class T, class U, class W>
  Sformatter(T const &t, U const &u, Manipulator m, W const &w)
  {
    resetStream();

    stream << t << u << m << '\0';

    mostRecentValue_ = stream.str().c_str();
  }

  template<class U, class W>
  Sformatter(Manipulator m1, U const &u, Manipulator m, W const &w)
  {
    resetStream();

    stream << m1 << u << m << '\0';

    mostRecentValue_ = stream.str().c_str();
  }


/// 5 parm calls //////////////////////////////////////////////


  template<class T, class U, class V, class W, class X>
  Sformatter(T const &t, U const &u, V const &v, W const &w, X const &x)
  {
    resetStream();

    stream << t << u << v << w << x << '\0';

    mostRecentValue_ = stream.str().c_str();
  }


  template<class U, class V, class W, class X>
  Sformatter(Manipulator m, U const &u, V const &v, W const &w, X const &x)
  {
    resetStream();

    stream << m << u << v << w << x << '\0';

    mostRecentValue_ = stream.str().c_str();
  }

  template<class T, class V, class W, class X>
  Sformatter(T const &t, Manipulator m, V const &v, W const &w, X const &x)
  {
    resetStream();

    stream << t << m << v << w << x << '\0';

    mostRecentValue_ = stream.str().c_str();
  }


  template<class T, class U, class W, class X>
  Sformatter(T const &t, U const &u, Manipulator m, W const &w, X const &x)
  {
    resetStream();

    stream << t << u << m << w << x << '\0';

    mostRecentValue_ = stream.str().c_str();
  }

  template<class T, class U, class V, class X>
  Sformatter(T const &t, U const &u, V const &v, Manipulator m, X const &x)
  {
    resetStream();

    stream << t << u << v << m << x << '\0';

    mostRecentValue_ = stream.str().c_str();
  }


  template<class U, class W, class X>
  Sformatter(Manipulator m1, U const &u, Manipulator m2, W const &w, X const &x)
  {
    resetStream();

    stream << m1 << u << m2 << w << x << '\0';

    mostRecentValue_ = stream.str().c_str();
  }

  template<class U, class V, class X>
  Sformatter(Manipulator m1, U const &u, V const &v, Manipulator m2, X const &x)
  {
    resetStream();

    stream << m1 << u << v << m2 << x << '\0';

    mostRecentValue_ = stream.str().c_str();
  }


};

std::stringstream       Sformatter<void>::stream;
std::string             Sformatter<void>::mostRecentValue_;
std::ios_base::fmtflags Sformatter<void>::formatFlags_ = stream.flags();  // get defaults



#endif
